@@ -36,6 +36,9 @@
 /* Get validator. */
 #include "validate.h"
 
+/* Get c_isalpha. */
+#include "c-ctype.h"
+
 static char *
 unescape (const char *str, size_t len)
 {
@@ -79,9 +82,9 @@ scram_parse_client_first (const char *str, size_t len,
   if (strnlen (str, len) < 10)
     return -1;
 
-  if (len == 0 || *str != 'n')
-      /* FIXME parse non-'n' cbflags */
-    return -1;
+  if (len == 0 || (*str != 'n' && *str != 'y'))
+      /* FIXME support channel bindings */
+      return -1;
   cf->cbflag = *str;
   str++, len--;
 
@@ -377,12 +380,35 @@ scram_parse_client_final (const char *str, size_t len,
     len -= l;
   }
 
-  /* FIXME check that any extension fields follow valid syntax. */
-
   if (len == 0 || *str != ',')
     return -1;
   str++, len--;
 
+  /* Ignore extensions. */
+  while (len > 0 && c_isalpha (*str) && *str != 'p')
+    {
+      const char *p;
+      size_t l;
+
+      str++, len--;
+
+      if (len == 0 || *str != '=')
+	return -1;
+      str++, len--;
+
+      p = memchr (str, ',', len);
+      if (!p)
+	return -1;
+      p++;
+
+      l = p - str;
+      if (len < l)
+	return -1;
+
+      str = p;
+      len -= l;
+    }
+
   if (len == 0 || *str != 'p')
     return -1;
   str++, len--;
