@@ -1,5 +1,5 @@
 /* xfinish.c --- Finish libgsasl session.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -32,6 +32,9 @@
 void
 gsasl_finish (Gsasl_session * sctx)
 {
+  if (sctx == NULL)
+    return;
+
   if (sctx->clientp)
     {
       if (sctx->mech && sctx->mech->client.finish)
