@@ -141,7 +141,7 @@ doit (void)
 
   gsasl_callback_set (ctx, callback);
 
-  for (i = 0; i <= 15; i++)
+  for (i = 0; i <= 21; i++)
     {
       bool server_first = (i % 2) == 0;
 
@@ -151,13 +151,15 @@ doit (void)
       res = gsasl_server_start (ctx, "SCRAM-SHA-1", &server);
       if (res != GSASL_OK)
 	{
-	  fail ("gsasl_init() failed (%d):\n%s\n", res, gsasl_strerror (res));
+	  fail ("gsasl_server_start() failed (%d):\n%s\n",
+		res, gsasl_strerror (res));
 	  return;
 	}
       res = gsasl_client_start (ctx, "SCRAM-SHA-1", &client);
       if (res != GSASL_OK)
 	{
-	  fail ("gsasl_init() failed (%d):\n%s\n", res, gsasl_strerror (res));
+	  fail ("gsasl_client_start() failed (%d):\n%s\n",
+		res, gsasl_strerror (res));
 	  return;
 	}
 
@@ -193,6 +195,29 @@ doit (void)
 	  return;
 	}
 
+      if (i == 16 || i == 17)
+	s1[0] = 'y';
+
+      if (i == 18)
+	{
+	  char *s;
+
+	  asprintf (&s, "%s,a=b", s1);
+	  gsasl_free (s1);
+	  s1 = s;
+	  s1len = strlen (s);
+	}
+
+      if (i == 20)
+	{
+	  char *s;
+
+	  asprintf (&s, "%s,a=b,b=c,c=d", s1);
+	  gsasl_free (s1);
+	  s1 = s;
+	  s1len = strlen (s);
+	}
+
       if (debug)
 	printf ("C: %.*s [%c]\n", s1len, s1, res == GSASL_OK ? 'O' : 'N');
 
@@ -221,6 +246,30 @@ doit (void)
 	  return;
 	}
 
+      if (i == 17)
+	memcpy (s1 + 2, "eS", 2);
+
+      if (i == 19 && s1len > 31)
+	{
+	  char *s;
+
+	  asprintf (&s, "%.*s,a=b,%s", s1len - 31, s1, s1 + s1len - 31 + 1);
+	  gsasl_free (s1);
+	  s1 = s;
+	  s1len = strlen (s);
+	}
+
+      if (i == 21 && s1len > 31)
+	{
+	  char *s;
+
+	  asprintf (&s, "%.*s,a=b,b=c,c=d,%s", s1len - 31, s1,
+		    s1 + s1len - 31 + 1);
+	  gsasl_free (s1);
+	  s1 = s;
+	  s1len = strlen (s);
+	}
+
       if (debug)
 	printf ("C: %.*s [%c]\n", s1len, s1, res == GSASL_OK ? 'O' : 'N');
 
@@ -228,6 +277,17 @@ doit (void)
 
       res = gsasl_step (server, s1, s1len, &s2, &s2len);
       gsasl_free (s1);
+      if (i >= 16 && i <= 21)
+	{
+	  if (res == GSASL_AUTHENTICATION_ERROR)
+	    {
+	      if (debug)
+		success ("Authentication failed expectedly\n");
+	      goto done;
+	    }
+	  else
+	    res = GSASL_AUTHENTICATION_ERROR;
+	}
       if (res != GSASL_OK)
 	{
 	  fail ("gsasl_step[%d](4) failed (%d):\n%s\n", i, res,
@@ -268,6 +328,7 @@ doit (void)
 	  fail ("Expected authzid? %d/%s\n", i, AUTHZID[i % N_AUTHZID]);
       }
 
+    done:
       if (debug)
 	printf ("\n");
 
