@@ -1,5 +1,5 @@
 /* simple.c --- Test the simple SASL mechanisms.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010  Simon Josefsson
  *
  * This file is part of GNU SASL.
  *
@@ -415,4 +415,8 @@ doit (void)
     }
 
   gsasl_done (ctx);
+
+  /* Sanity check interfaces. */
+  gsasl_finish (NULL);
+  gsasl_done (NULL);
 }
